// Code generated by protoc-gen-go. DO NOT EDIT.
// source: xds/core/v3/resource_name.proto

package xds_core_v3

import (
	fmt "fmt"
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	proto "github.com/golang/protobuf/proto"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type ResourceName struct {
	Id                   string         `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	Authority            string         `protobuf:"bytes,2,opt,name=authority,proto3" json:"authority,omitempty"`
	ResourceType         string         `protobuf:"bytes,3,opt,name=resource_type,json=resourceType,proto3" json:"resource_type,omitempty"`
	Context              *ContextParams `protobuf:"bytes,4,opt,name=context,proto3" json:"context,omitempty"`
	XXX_NoUnkeyedLiteral struct{}       `json:"-"`
	XXX_unrecognized     []byte         `json:"-"`
	XXX_sizecache        int32          `json:"-"`
}

func (m *ResourceName) Reset()         { *m = ResourceName{} }
func (m *ResourceName) String() string { return proto.CompactTextString(m) }
func (*ResourceName) ProtoMessage()    {}
func (*ResourceName) Descriptor() ([]byte, []int) {
	return fileDescriptor_142e5d243416c11e, []int{0}
}

func (m *ResourceName) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResourceName.Unmarshal(m, b)
}
func (m *ResourceName) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResourceName.Marshal(b, m, deterministic)
}
func (m *ResourceName) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResourceName.Merge(m, src)
}
func (m *ResourceName) XXX_Size() int {
	return xxx_messageInfo_ResourceName.Size(m)
}
func (m *ResourceName) XXX_DiscardUnknown() {
	xxx_messageInfo_ResourceName.DiscardUnknown(m)
}

var xxx_messageInfo_ResourceName proto.InternalMessageInfo

func (m *ResourceName) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *ResourceName) GetAuthority() string {
	if m != nil {
		return m.Authority
	}
	return ""
}

func (m *ResourceName) GetResourceType() string {
	if m != nil {
		return m.ResourceType
	}
	return ""
}

func (m *ResourceName) GetContext() *ContextParams {
	if m != nil {
		return m.Context
	}
	return nil
}

func init() {
	proto.RegisterType((*ResourceName)(nil), "xds.core.v3.ResourceName")
}

func init() { proto.RegisterFile("xds/core/v3/resource_name.proto", fileDescriptor_142e5d243416c11e) }

var fileDescriptor_142e5d243416c11e = []byte{
	// 271 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x4c, 0x90, 0x31, 0x4e, 0xc3, 0x30,
	0x14, 0x86, 0xe5, 0x50, 0xb5, 0xd4, 0x2d, 0x08, 0xb2, 0x10, 0x05, 0x10, 0x11, 0x53, 0x07, 0x64,
	0x4b, 0x84, 0x89, 0x31, 0xec, 0x28, 0x8a, 0xd8, 0xab, 0xd7, 0xd8, 0xa2, 0x96, 0x48, 0x1c, 0xd9,
	0xcf, 0x51, 0xb2, 0x71, 0x14, 0xce, 0xc1, 0x09, 0x58, 0xb9, 0x0e, 0x13, 0x4a, 0xd2, 0x40, 0x36,
	0xeb, 0xfd, 0x9f, 0xed, 0xff, 0x7b, 0xf4, 0xa6, 0x11, 0x96, 0xe7, 0xda, 0x48, 0x5e, 0xc7, 0xdc,
	0x48, 0xab, 0x9d, 0xc9, 0xe5, 0xb6, 0x84, 0x42, 0xb2, 0xca, 0x68, 0xd4, 0xfe, 0xaa, 0x11, 0x96,
	0x75, 0x00, 0xab, 0xe3, 0xf0, 0xda, 0x89, 0x0a, 0x38, 0x94, 0xa5, 0x46, 0x40, 0xa5, 0x4b, 0xcb,
	0x2d, 0x02, 0x3a, 0x3b, 0xb0, 0x61, 0x34, 0x7d, 0x2c, 0xd7, 0x25, 0xca, 0x06, 0xb7, 0x15, 0x18,
	0x28, 0x46, 0xe2, 0xa2, 0x86, 0x37, 0x25, 0x00, 0x25, 0x1f, 0x0f, 0x43, 0x70, 0xfb, 0x41, 0xe8,
	0x3a, 0x3b, 0x7c, 0xff, 0x0c, 0x85, 0xf4, 0x4f, 0xa9, 0xa7, 0x44, 0x40, 0x22, 0xb2, 0x59, 0x66,
	0x9e, 0x12, 0xfe, 0x15, 0x5d, 0x82, 0xc3, 0xbd, 0x36, 0x0a, 0xdb, 0xc0, 0xeb, 0xc7, 0xff, 0x03,
	0xff, 0x8e, 0x9e, 0xfc, 0x95, 0xc7, 0xb6, 0x92, 0xc1, 0x51, 0x47, 0x24, 0x8b, 0x9f, 0x64, 0x66,
	0xbc, 0x33, 0x92, 0xad, 0xc7, 0xf4, 0xa5, 0xad, 0xa4, 0xff, 0x40, 0x17, 0x87, 0x76, 0xc1, 0x2c,
	0x22, 0x9b, 0xd5, 0x7d, 0xc8, 0x26, 0x96, 0xec, 0x69, 0xc8, 0xd2, 0xbe, 0x78, 0x36, 0xa2, 0xc9,
	0xe3, 0xe7, 0xfb, 0xd7, 0xf7, 0xdc, 0x3b, 0x26, 0xf4, 0x32, 0xd7, 0x05, 0x7b, 0x55, 0xb8, 0x77,
	0x3b, 0xd6, 0x6d, 0x64, 0x7a, 0x3b, 0x39, 0x9f, 0x6a, 0xa4, 0x9d, 0x5c, 0x4a, 0x76, 0xf3, 0xde,
	0x32, 0xfe, 0x0d, 0x00, 0x00, 0xff, 0xff, 0x8a, 0x99, 0xd1, 0xf6, 0x6f, 0x01, 0x00, 0x00,
}
