package version

// Copyright (c) Microsoft Corporation. All rights reserved.
// Licensed under the MIT License. See License.txt in the project root for license information.

// Number contains the semantic version of this SDK.
const Number = "v57.4.0"
