package pipeline

const (
	// UserAgent is the string to be used in the user agent string when making requests.
	UserAgent = "azure-pipeline-go/" + Version

	// Version is the semantic version (see http://semver.org) of the pipeline package.
	Version = "0.2.1"
)
