// Code generated by protoc-gen-go. DO NOT EDIT.
// source: xds/core/v3/resource_locator.proto

package xds_core_v3

import (
	fmt "fmt"
	_ "github.com/cncf/xds/go/udpa/annotations"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	proto "github.com/golang/protobuf/proto"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type ResourceLocator_Scheme int32

const (
	ResourceLocator_XDSTP ResourceLocator_Scheme = 0
	ResourceLocator_HTTP  ResourceLocator_Scheme = 1
	ResourceLocator_FILE  ResourceLocator_Scheme = 2
)

var ResourceLocator_Scheme_name = map[int32]string{
	0: "XDSTP",
	1: "HTTP",
	2: "FILE",
}

var ResourceLocator_Scheme_value = map[string]int32{
	"XDSTP": 0,
	"HTTP":  1,
	"FILE":  2,
}

func (x ResourceLocator_Scheme) String() string {
	return proto.EnumName(ResourceLocator_Scheme_name, int32(x))
}

func (ResourceLocator_Scheme) EnumDescriptor() ([]byte, []int) {
	return fileDescriptor_eb09b3779eaf3665, []int{0, 0}
}

type ResourceLocator struct {
	Scheme       ResourceLocator_Scheme `protobuf:"varint,1,opt,name=scheme,proto3,enum=xds.core.v3.ResourceLocator_Scheme" json:"scheme,omitempty"`
	Id           string                 `protobuf:"bytes,2,opt,name=id,proto3" json:"id,omitempty"`
	Authority    string                 `protobuf:"bytes,3,opt,name=authority,proto3" json:"authority,omitempty"`
	ResourceType string                 `protobuf:"bytes,4,opt,name=resource_type,json=resourceType,proto3" json:"resource_type,omitempty"`
	// Types that are valid to be assigned to ContextParamSpecifier:
	//	*ResourceLocator_ExactContext
	ContextParamSpecifier isResourceLocator_ContextParamSpecifier `protobuf_oneof:"context_param_specifier"`
	Directives            []*ResourceLocator_Directive            `protobuf:"bytes,6,rep,name=directives,proto3" json:"directives,omitempty"`
	XXX_NoUnkeyedLiteral  struct{}                                `json:"-"`
	XXX_unrecognized      []byte                                  `json:"-"`
	XXX_sizecache         int32                                   `json:"-"`
}

func (m *ResourceLocator) Reset()         { *m = ResourceLocator{} }
func (m *ResourceLocator) String() string { return proto.CompactTextString(m) }
func (*ResourceLocator) ProtoMessage()    {}
func (*ResourceLocator) Descriptor() ([]byte, []int) {
	return fileDescriptor_eb09b3779eaf3665, []int{0}
}

func (m *ResourceLocator) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResourceLocator.Unmarshal(m, b)
}
func (m *ResourceLocator) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResourceLocator.Marshal(b, m, deterministic)
}
func (m *ResourceLocator) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResourceLocator.Merge(m, src)
}
func (m *ResourceLocator) XXX_Size() int {
	return xxx_messageInfo_ResourceLocator.Size(m)
}
func (m *ResourceLocator) XXX_DiscardUnknown() {
	xxx_messageInfo_ResourceLocator.DiscardUnknown(m)
}

var xxx_messageInfo_ResourceLocator proto.InternalMessageInfo

func (m *ResourceLocator) GetScheme() ResourceLocator_Scheme {
	if m != nil {
		return m.Scheme
	}
	return ResourceLocator_XDSTP
}

func (m *ResourceLocator) GetId() string {
	if m != nil {
		return m.Id
	}
	return ""
}

func (m *ResourceLocator) GetAuthority() string {
	if m != nil {
		return m.Authority
	}
	return ""
}

func (m *ResourceLocator) GetResourceType() string {
	if m != nil {
		return m.ResourceType
	}
	return ""
}

type isResourceLocator_ContextParamSpecifier interface {
	isResourceLocator_ContextParamSpecifier()
}

type ResourceLocator_ExactContext struct {
	ExactContext *ContextParams `protobuf:"bytes,5,opt,name=exact_context,json=exactContext,proto3,oneof"`
}

func (*ResourceLocator_ExactContext) isResourceLocator_ContextParamSpecifier() {}

func (m *ResourceLocator) GetContextParamSpecifier() isResourceLocator_ContextParamSpecifier {
	if m != nil {
		return m.ContextParamSpecifier
	}
	return nil
}

func (m *ResourceLocator) GetExactContext() *ContextParams {
	if x, ok := m.GetContextParamSpecifier().(*ResourceLocator_ExactContext); ok {
		return x.ExactContext
	}
	return nil
}

func (m *ResourceLocator) GetDirectives() []*ResourceLocator_Directive {
	if m != nil {
		return m.Directives
	}
	return nil
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*ResourceLocator) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*ResourceLocator_ExactContext)(nil),
	}
}

type ResourceLocator_Directive struct {
	// Types that are valid to be assigned to Directive:
	//	*ResourceLocator_Directive_Alt
	//	*ResourceLocator_Directive_Entry
	Directive            isResourceLocator_Directive_Directive `protobuf_oneof:"directive"`
	XXX_NoUnkeyedLiteral struct{}                              `json:"-"`
	XXX_unrecognized     []byte                                `json:"-"`
	XXX_sizecache        int32                                 `json:"-"`
}

func (m *ResourceLocator_Directive) Reset()         { *m = ResourceLocator_Directive{} }
func (m *ResourceLocator_Directive) String() string { return proto.CompactTextString(m) }
func (*ResourceLocator_Directive) ProtoMessage()    {}
func (*ResourceLocator_Directive) Descriptor() ([]byte, []int) {
	return fileDescriptor_eb09b3779eaf3665, []int{0, 0}
}

func (m *ResourceLocator_Directive) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_ResourceLocator_Directive.Unmarshal(m, b)
}
func (m *ResourceLocator_Directive) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_ResourceLocator_Directive.Marshal(b, m, deterministic)
}
func (m *ResourceLocator_Directive) XXX_Merge(src proto.Message) {
	xxx_messageInfo_ResourceLocator_Directive.Merge(m, src)
}
func (m *ResourceLocator_Directive) XXX_Size() int {
	return xxx_messageInfo_ResourceLocator_Directive.Size(m)
}
func (m *ResourceLocator_Directive) XXX_DiscardUnknown() {
	xxx_messageInfo_ResourceLocator_Directive.DiscardUnknown(m)
}

var xxx_messageInfo_ResourceLocator_Directive proto.InternalMessageInfo

type isResourceLocator_Directive_Directive interface {
	isResourceLocator_Directive_Directive()
}

type ResourceLocator_Directive_Alt struct {
	Alt *ResourceLocator `protobuf:"bytes,1,opt,name=alt,proto3,oneof"`
}

type ResourceLocator_Directive_Entry struct {
	Entry string `protobuf:"bytes,2,opt,name=entry,proto3,oneof"`
}

func (*ResourceLocator_Directive_Alt) isResourceLocator_Directive_Directive() {}

func (*ResourceLocator_Directive_Entry) isResourceLocator_Directive_Directive() {}

func (m *ResourceLocator_Directive) GetDirective() isResourceLocator_Directive_Directive {
	if m != nil {
		return m.Directive
	}
	return nil
}

func (m *ResourceLocator_Directive) GetAlt() *ResourceLocator {
	if x, ok := m.GetDirective().(*ResourceLocator_Directive_Alt); ok {
		return x.Alt
	}
	return nil
}

func (m *ResourceLocator_Directive) GetEntry() string {
	if x, ok := m.GetDirective().(*ResourceLocator_Directive_Entry); ok {
		return x.Entry
	}
	return ""
}

// XXX_OneofWrappers is for the internal use of the proto package.
func (*ResourceLocator_Directive) XXX_OneofWrappers() []interface{} {
	return []interface{}{
		(*ResourceLocator_Directive_Alt)(nil),
		(*ResourceLocator_Directive_Entry)(nil),
	}
}

func init() {
	proto.RegisterEnum("xds.core.v3.ResourceLocator_Scheme", ResourceLocator_Scheme_name, ResourceLocator_Scheme_value)
	proto.RegisterType((*ResourceLocator)(nil), "xds.core.v3.ResourceLocator")
	proto.RegisterType((*ResourceLocator_Directive)(nil), "xds.core.v3.ResourceLocator.Directive")
}

func init() { proto.RegisterFile("xds/core/v3/resource_locator.proto", fileDescriptor_eb09b3779eaf3665) }

var fileDescriptor_eb09b3779eaf3665 = []byte{
	// 481 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x7c, 0x92, 0x41, 0x6f, 0xd3, 0x30,
	0x14, 0xc7, 0xeb, 0xb4, 0x0d, 0x8d, 0xbb, 0x8d, 0xc8, 0x42, 0x2c, 0x84, 0x22, 0xa2, 0x82, 0xa0,
	0x12, 0x34, 0x99, 0xd2, 0x03, 0x02, 0x89, 0xc3, 0xcc, 0x36, 0x15, 0x69, 0x87, 0x28, 0xeb, 0x01,
	0xc1, 0x20, 0xf2, 0x12, 0x43, 0x2d, 0xb5, 0x71, 0x64, 0x3b, 0x55, 0xcb, 0x01, 0x21, 0x4e, 0x9c,
	0xf8, 0x40, 0x7c, 0x02, 0xae, 0x7c, 0x1b, 0xd4, 0x13, 0x4a, 0xd2, 0x76, 0xdd, 0x0e, 0xbb, 0x59,
	0xef, 0xfd, 0xde, 0xd3, 0xff, 0xfd, 0xfd, 0x87, 0xdd, 0x79, 0x22, 0xbd, 0x98, 0x0b, 0xea, 0xcd,
	0x06, 0x9e, 0xa0, 0x92, 0xe7, 0x22, 0xa6, 0xd1, 0x84, 0xc7, 0x44, 0x71, 0xe1, 0x66, 0x82, 0x2b,
	0x8e, 0xda, 0xf3, 0x44, 0xba, 0x05, 0xe3, 0xce, 0x06, 0xf6, 0x83, 0x3c, 0xc9, 0x88, 0x47, 0xd2,
	0x94, 0x2b, 0xa2, 0x18, 0x4f, 0xa5, 0x27, 0x15, 0x51, 0xb9, 0xac, 0x58, 0xdb, 0xd9, 0xde, 0x17,
	0xf3, 0x54, 0xd1, 0xb9, 0x8a, 0x32, 0x22, 0xc8, 0x74, 0x4d, 0xec, 0xcf, 0xc8, 0x84, 0x25, 0x44,
	0x51, 0x6f, 0xfd, 0xa8, 0x1a, 0xdd, 0x5f, 0x0d, 0x78, 0x3b, 0x5c, 0x29, 0x38, 0xad, 0x04, 0xa0,
	0x63, 0xa8, 0xcb, 0x78, 0x4c, 0xa7, 0xd4, 0x02, 0x0e, 0xe8, 0xed, 0xf9, 0x8f, 0xdc, 0x2d, 0x2d,
	0xee, 0x35, 0xda, 0x3d, 0x2b, 0x51, 0xdc, 0x5a, 0xe2, 0xe6, 0x0f, 0xa0, 0x99, 0x20, 0x5c, 0x0d,
	0xa3, 0x3d, 0xa8, 0xb1, 0xc4, 0xd2, 0x1c, 0xd0, 0x33, 0x42, 0x8d, 0x25, 0xa8, 0x03, 0x0d, 0x92,
	0xab, 0x31, 0x17, 0x4c, 0x2d, 0xac, 0x7a, 0x59, 0xbe, 0x2c, 0xa0, 0xe7, 0x70, 0x77, 0xe3, 0x84,
	0x5a, 0x64, 0xd4, 0x6a, 0x14, 0x04, 0xbe, 0xb5, 0xc4, 0x0d, 0x51, 0x6c, 0xdd, 0x59, 0x77, 0x47,
	0x8b, 0x8c, 0xa2, 0x43, 0xb8, 0x4b, 0xe7, 0x24, 0x56, 0xd1, 0xea, 0x5a, 0xab, 0xe9, 0x80, 0x5e,
	0xdb, 0xb7, 0xaf, 0x28, 0x7d, 0x53, 0xf5, 0x82, 0xd2, 0x88, 0x61, 0x2d, 0xdc, 0x29, 0x47, 0x56,
	0x55, 0x74, 0x02, 0x61, 0xc2, 0x04, 0x8d, 0x15, 0x9b, 0x51, 0x69, 0xe9, 0x4e, 0xbd, 0xd7, 0xf6,
	0x9f, 0xdc, 0x78, 0xe9, 0xd1, 0x1a, 0x0f, 0xb7, 0x26, 0xed, 0x9f, 0x00, 0x1a, 0x9b, 0x0e, 0x3a,
	0x80, 0x75, 0x32, 0x51, 0xa5, 0x71, 0x6d, 0xbf, 0x73, 0xd3, 0xba, 0x61, 0x2d, 0x2c, 0x50, 0xf4,
	0x02, 0x36, 0x69, 0xaa, 0xc4, 0xa2, 0x72, 0x0a, 0x3f, 0x5c, 0xe2, 0x8e, 0xb0, 0x4d, 0xe0, 0xdf,
	0xfd, 0xf4, 0xe1, 0xa0, 0xff, 0x92, 0xf4, 0xbf, 0x1e, 0xf6, 0xdf, 0x47, 0xe7, 0xfd, 0x73, 0xd7,
	0xfb, 0xf6, 0xea, 0xe3, 0xb3, 0xc7, 0xc3, 0x5a, 0x58, 0xf1, 0xd8, 0x84, 0xc6, 0x46, 0x06, 0xaa,
	0xff, 0xc3, 0xa0, 0xfb, 0x14, 0xea, 0xd5, 0x6f, 0x20, 0x03, 0x36, 0xdf, 0x1d, 0x9d, 0x8d, 0x02,
	0xb3, 0x86, 0x5a, 0xb0, 0x31, 0x1c, 0x8d, 0x02, 0x13, 0x14, 0xaf, 0x93, 0xb7, 0xa7, 0xc7, 0xa6,
	0x86, 0xef, 0xc1, 0xfd, 0x2b, 0x31, 0x89, 0x64, 0x46, 0x63, 0xf6, 0x99, 0x51, 0x81, 0x5f, 0xff,
	0xfe, 0xfe, 0xe7, 0xaf, 0xae, 0xb5, 0x00, 0xbc, 0x1f, 0xf3, 0xa9, 0xfb, 0x85, 0xa9, 0x71, 0x7e,
	0xe1, 0x16, 0xf9, 0xdb, 0x3e, 0x06, 0xdf, 0xb9, 0x76, 0x4d, 0x50, 0xa4, 0x29, 0x00, 0x17, 0x7a,
	0x19, 0xab, 0xc1, 0xff, 0x00, 0x00, 0x00, 0xff, 0xff, 0x46, 0x23, 0x9e, 0xd8, 0xe3, 0x02, 0x00,
	0x00,
}
