// Code generated by protoc-gen-go. DO NOT EDIT.
// source: udpa/annotations/security.proto

package udpa_annotations

import (
	fmt "fmt"
	_ "github.com/envoyproxy/protoc-gen-validate/validate"
	proto "github.com/golang/protobuf/proto"
	descriptor "github.com/golang/protobuf/protoc-gen-go/descriptor"
	_ "github.com/golang/protobuf/ptypes/any"
	math "math"
)

// Reference imports to suppress errors if they are not otherwise used.
var _ = proto.Marshal
var _ = fmt.Errorf
var _ = math.Inf

// This is a compile-time assertion to ensure that this generated file
// is compatible with the proto package it is being compiled against.
// A compilation error at this line likely means your copy of the
// proto package needs to be updated.
const _ = proto.ProtoPackageIsVersion3 // please upgrade the proto package

type FieldSecurityAnnotation struct {
	ConfigureForUntrustedDownstream bool     `protobuf:"varint,1,opt,name=configure_for_untrusted_downstream,json=configureForUntrustedDownstream,proto3" json:"configure_for_untrusted_downstream,omitempty"`
	ConfigureForUntrustedUpstream   bool     `protobuf:"varint,2,opt,name=configure_for_untrusted_upstream,json=configureForUntrustedUpstream,proto3" json:"configure_for_untrusted_upstream,omitempty"`
	XXX_NoUnkeyedLiteral            struct{} `json:"-"`
	XXX_unrecognized                []byte   `json:"-"`
	XXX_sizecache                   int32    `json:"-"`
}

func (m *FieldSecurityAnnotation) Reset()         { *m = FieldSecurityAnnotation{} }
func (m *FieldSecurityAnnotation) String() string { return proto.CompactTextString(m) }
func (*FieldSecurityAnnotation) ProtoMessage()    {}
func (*FieldSecurityAnnotation) Descriptor() ([]byte, []int) {
	return fileDescriptor_43b150013eccfb0a, []int{0}
}

func (m *FieldSecurityAnnotation) XXX_Unmarshal(b []byte) error {
	return xxx_messageInfo_FieldSecurityAnnotation.Unmarshal(m, b)
}
func (m *FieldSecurityAnnotation) XXX_Marshal(b []byte, deterministic bool) ([]byte, error) {
	return xxx_messageInfo_FieldSecurityAnnotation.Marshal(b, m, deterministic)
}
func (m *FieldSecurityAnnotation) XXX_Merge(src proto.Message) {
	xxx_messageInfo_FieldSecurityAnnotation.Merge(m, src)
}
func (m *FieldSecurityAnnotation) XXX_Size() int {
	return xxx_messageInfo_FieldSecurityAnnotation.Size(m)
}
func (m *FieldSecurityAnnotation) XXX_DiscardUnknown() {
	xxx_messageInfo_FieldSecurityAnnotation.DiscardUnknown(m)
}

var xxx_messageInfo_FieldSecurityAnnotation proto.InternalMessageInfo

func (m *FieldSecurityAnnotation) GetConfigureForUntrustedDownstream() bool {
	if m != nil {
		return m.ConfigureForUntrustedDownstream
	}
	return false
}

func (m *FieldSecurityAnnotation) GetConfigureForUntrustedUpstream() bool {
	if m != nil {
		return m.ConfigureForUntrustedUpstream
	}
	return false
}

var E_Security = &proto.ExtensionDesc{
	ExtendedType:  (*descriptor.FieldOptions)(nil),
	ExtensionType: (*FieldSecurityAnnotation)(nil),
	Field:         11122993,
	Name:          "udpa.annotations.security",
	Tag:           "bytes,11122993,opt,name=security",
	Filename:      "udpa/annotations/security.proto",
}

func init() {
	proto.RegisterType((*FieldSecurityAnnotation)(nil), "udpa.annotations.FieldSecurityAnnotation")
	proto.RegisterExtension(E_Security)
}

func init() { proto.RegisterFile("udpa/annotations/security.proto", fileDescriptor_43b150013eccfb0a) }

var fileDescriptor_43b150013eccfb0a = []byte{
	// 276 bytes of a gzipped FileDescriptorProto
	0x1f, 0x8b, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xff, 0x74, 0x90, 0xb1, 0x4a, 0x03, 0x31,
	0x18, 0xc7, 0xb9, 0x82, 0xe5, 0x88, 0x8b, 0xdc, 0xd2, 0x5a, 0x38, 0x7a, 0x74, 0xaa, 0x4b, 0x02,
	0xba, 0xb9, 0x29, 0x52, 0x07, 0x07, 0xa1, 0xd2, 0xf9, 0x48, 0x2f, 0xb9, 0x23, 0x70, 0xe6, 0x0b,
	0xc9, 0x17, 0xc5, 0xcd, 0xa7, 0x71, 0x15, 0x57, 0x9f, 0xc0, 0xd5, 0x57, 0xf1, 0x09, 0xc4, 0x5c,
	0xee, 0x90, 0x6a, 0xb7, 0x84, 0xef, 0xf7, 0xfd, 0x92, 0xff, 0x9f, 0xcc, 0xbd, 0x30, 0x9c, 0x71,
	0xad, 0x01, 0x39, 0x2a, 0xd0, 0x8e, 0x39, 0x59, 0x79, 0xab, 0xf0, 0x89, 0x1a, 0x0b, 0x08, 0xd9,
	0xd1, 0x0f, 0x40, 0x7f, 0x01, 0xb3, 0xfc, 0xef, 0x0a, 0x72, 0xf4, 0xae, 0x5b, 0x98, 0x1d, 0x37,
	0x00, 0x4d, 0x2b, 0x59, 0xb8, 0x6d, 0x7d, 0xcd, 0xb8, 0x8e, 0xae, 0x59, 0xb1, 0x3b, 0x12, 0xd2,
	0x55, 0x56, 0x19, 0x04, 0x1b, 0x89, 0xc9, 0x03, 0x6f, 0x95, 0xe0, 0x28, 0x59, 0x7f, 0xe8, 0x06,
	0x8b, 0xd7, 0x84, 0x4c, 0x56, 0x4a, 0xb6, 0xe2, 0x2e, 0x7e, 0xef, 0x62, 0x78, 0x3f, 0xbb, 0x21,
	0x8b, 0x0a, 0x74, 0xad, 0x1a, 0x6f, 0x65, 0x59, 0x83, 0x2d, 0xbd, 0x46, 0xeb, 0x1d, 0x4a, 0x51,
	0x0a, 0x78, 0xd4, 0x0e, 0xad, 0xe4, 0xf7, 0xd3, 0xa4, 0x48, 0x96, 0xe9, 0x7a, 0x3e, 0x90, 0x2b,
	0xb0, 0x9b, 0x9e, 0xbb, 0x1a, 0xb0, 0xec, 0x9a, 0x14, 0xfb, 0x64, 0xde, 0x44, 0xd5, 0x28, 0xa8,
	0xf2, 0x7f, 0x55, 0x9b, 0x08, 0x9d, 0x37, 0x24, 0xed, 0xab, 0xcc, 0x72, 0xda, 0x25, 0xa7, 0x7d,
	0x72, 0x1a, 0xb2, 0xdc, 0x9a, 0xd0, 0xdf, 0xf4, 0xed, 0xeb, 0xe5, 0xa0, 0x48, 0x96, 0x87, 0xa7,
	0x27, 0x74, 0xb7, 0x6d, 0xba, 0x27, 0xf4, 0x7a, 0x90, 0x5f, 0xa6, 0xef, 0xcf, 0x1f, 0x9f, 0xe3,
	0x51, 0x9a, 0x6c, 0xc7, 0x41, 0x7f, 0xf6, 0x1d, 0x00, 0x00, 0xff, 0xff, 0xc9, 0xf3, 0x6a, 0x56,
	0xd5, 0x01, 0x00, 0x00,
}
